import logging
from chibi_elasticsearch.config import get_all_hosts_in_config


logger = logging.getLogger( 'chibi_elasticsearch.unittests.vcr' )


try:
    from vcr_unittest import VCRTestCase
except ImportError:
    logger.error(
        "No se puede importar los vcrs se nesesita instalar las "
        "dependencias 'pip install chibi_elasticsearch[vcr]'" )
    raise


class Chibi_elastic_vcr( VCRTestCase ):
    """
    clase atajo para que los vcr ignoren los hosts de elasticsearch
    """
    def _get_vcr_kwargs( self, **kw ):
        kw = super()._get_vcr_kwargs( **kw )
        kw[ 'ignore_hosts' ] = self.get_all_ignore_hosts()
        return kw

    def get_all_ignore_hosts( self ):
        return get_all_hosts_in_config()



from chibi.config import configuration
from elasticsearch_dsl.connections import connections


def build_index_name( name, app_name=None, ):
    """
    crea el nombre del indice para producion o para pruebas

    Para asirnar que es una prueba se usa la configuracion de chibi

    Parameters
    ----------
    name: str
    app_name: str
        por default toma el nombre del projecto

    Example
    -------
    >>>from chibi.config import configuration
    >>>configuration.elasticsearch.test_app = True
    >>>assert build_index_name( 'hello' ) == 'test__hello'
    >>>configuration.elasticsearch.test_app = False
    >>>assert build_index_name( 'hello' ) == 'hello'
    """
    if app_name:
        result = f"{app_name}__{name}"
    else:
        result = name

    is_test = bool( configuration.elasticsearch.test_app )
    if is_test:
        return f"test__{result}"
    return result


def list_indices():
    es = connections.get_connection()
    return es.cat.indices( format='json' )


def index_exists( model_class ):
    return model_class._index.exists()


def create_index_if_not_exists( model_class ):
    if not index_exists( model_class ):
        model_class.init()



import datetime
import logging

from elasticsearch_dsl import Document, field


logger = logging.getLogger( 'chibi_django.models' )


class Chibi_model( Document ):
    """
    clase basica para usar elastisearch

    Arguments
    ---------
    create_at: datetime.datetime
        fecha en la que se agrebo el modelo
    update_at: datetime.datetime
        fecha en la que se actualizo el modelo
    """
    create_at = field.Date()
    update_at = field.Date()

    @property
    def pk( self ):
        return getattr( self.meta, 'id', None )

    def save( self, *args, **kw ):
        if not getattr( self.meta, 'id', False ):
            if getattr( self, 'id', False ):
                self.meta.id = self.id
                del self.id
            if not getattr( self, 'create_at' ):
                self.create_at = datetime.datetime.now( datetime.UTC )
        self.update_at = datetime.datetime.now( datetime.UTC )
        return super().save( *args, **kw )



#!/usr/bin/env python3
import logging
from chibi.config import configuration
from chibi.snippet.is_type import is_like_list
from elasticsearch_dsl.connections import connections


logger = logging.getLogger( 'chibi_elasticsearch.config' )


def load_elasticsearch_config():
    connections.configure( **configuration.elasticsearch.connections )


def review_elasticsearch_config():
    default = connections.get_connection()
    if not default.transport.hosts:
        logger.error(
            "no se detecto configuracion de elasticsearch, "
            "ejemplo de configuracion\n"
            "from elasticsearch_dsl.connections import connections"
            "connections.configure( 'default': { 'hosts': 'localhost:80' } )"
        )
        return False
    return True


def _get_all_hosts_in_config():
    keys = connections._kwargs.keys()
    conns = list(
        connections.get_connection( alias=alias ) for alias in keys
    )
    hosts = ( c.transport.hosts for c in conns )
    for host in hosts:
        if is_like_list( host ):
            for h in host:
                if isinstance( h, dict ):
                    yield h[ 'host' ]
                else:
                    yield h
        elif isinstance( host, dict ):
            yield host[ 'host' ]
        else:
            yield host


def get_all_hosts_in_config():
    """
    regresa la lista de todos los hosts
    de elasticsearch_dsl.connections.connections
    """
    return list( _get_all_hosts_in_config() )



from unittest.mock import patch
import functools
import logging


logger = logging.getLogger( 'chibi_elasticsearch.unittests' )


def patch_doc_save( func, target='elasticsearch_dsl.Document.save' ):
    """
    decorador para parchear la funcion save de los documentos de
    elasticseach dls
    """
    @functools.wraps( func )
    def wrapper( *args, **kw ):
        patcher = patch( target )
        with patcher as mock:
            func( *args, mock, **kw )
    return wrapper



from elasticsearch_dsl import analyzer, tokenizer


name = analyzer(
    'name',
    tokenizer=tokenizer( 'trigram', 'nGram', min_gram=3, max_gram=4 ),
    filter=[ "lowercase", ],
)

name_space = analyzer(
    'name_space',
    tokenizer='whitespace',
    filter=[ "lowercase", ],
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [ 'chibi>=0.17.0', "elasticsearch-dsl>7,<8" ]
vcr_requirements = [ 'vcrpy-unittest>=0.1.7' ]


setup(
    author="dem4ply",
    author_email='dem4ply@gmail.com',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: Public Domain',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="Package with templates and snippets for elasticsearch",
    install_requires=requirements,
    extras_require={
        'vcr': vcr_requirements,
    },
    license="WTFPL",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='chibi_elasticsearch',
    name='chibi_elasticsearch',
    packages=find_packages(include=['chibi_elasticsearch', 'chibi_elasticsearch.*']),
    url='https://github.com/dem4ply/chibi_elasticsearch',
    version='1.3.0',
    zip_safe=False,
)
